@@ -23,6 +23,7 @@ namespace boost {
 #include <Wt/WCssStyleSheet.h>
 #include <Wt/WEvent.h>
 #include <Wt/WJavaScriptPreamble.h>
+#include <Wt/WJavaScriptSlot.h>
 #include <Wt/WLocale.h>
 #include <Wt/WMessageResourceBundle.h>
 #include <Wt/WSignal.h>
@@ -39,6 +40,12 @@ struct WtLibVersion { };
 extern WT_API const WtLibVersion WT_INCLUDED_VERSION;
 #endif
 
+#ifdef WT_TARGET_JAVA
+struct AutoCloseable {
+  virtual void close() = 0;
+};
+#endif // WT_TARGET_JAVA
+
 class WApplication;
 class WCombinedLocalizedStrings;
 class WContainerWidget;
@@ -54,16 +61,13 @@ class RootContainer;
 class UpdateLockImpl;
 class SoundManager;
 
-#ifndef WT_TARGET_JAVA
 /*! \brief Typedef for a function that creates WApplication objects.
  *
  * \sa WRun()
  *
  * \relates WApplication
  */
-  typedef std::function<std::unique_ptr<WApplication> (const WEnvironment&)>
-    ApplicationCreator;
-#endif // !WT_TARGET_JAVA
+typedef std::function<std::unique_ptr<WApplication> (const WEnvironment&)> ApplicationCreator;
 
 #ifdef WT_TARGET_JAVA
 /*! \brief An HTML Meta Header
@@ -206,13 +210,11 @@ public:
 class WT_API WApplication : public WObject
 {
 public:
-#ifndef WT_TARGET_JAVA
   /*! \brief Typedef for a function that creates WApplication objects.
    *
    * \sa WRun()
    */
   typedef Wt::ApplicationCreator ApplicationCreator;
-#endif // WT_TARGET_JAVA
 
   /*! \brief Creates a new application instance.
    *
@@ -541,6 +543,8 @@ public:
    * object, and this method returns localizedStrings() downcasted to
    * this type.
    *
+   * \throws WException when localizedStrings() is not a WMessageResourceBundle
+   *
    * \sa WString::tr(const char *key)
    */
   WMessageResourceBundle& messageResourceBundle();
@@ -575,8 +579,8 @@ public:
 
   /*! \brief Refreshes the application.
    *
-   * This lets the application to refresh its data, including strings
-   * from message-resource bundles. This done by propagating
+   * This lets the application refresh its data, including strings
+   * from message resource bundles. This is done by propagating
    * WWidget::refresh() through the widget hierarchy.
    *
    * This method is also called when the user hits the refresh (or
@@ -1265,7 +1269,7 @@ public:
    *         application and its widgets outside of the event loop.
    *
    * You can use this lock to manipulate widgets outside of the event
-   * loop. LabelOption::Inside the event loop (including events posted using
+   * loop. Inside the event loop (including events posted using
    * WServer::post()), this lock is already held by the library itself.
    *
    * The lock is recursive, so trying to take a lock, while already
@@ -1283,6 +1287,9 @@ public:
    */
 #endif // WT_TARGET_JAVA
   class WT_API UpdateLock
+#ifdef WT_TARGET_JAVA
+    : public AutoCloseable
+#endif // WT_TARGET_JAVA
   {
   public:
 #ifndef WT_TARGET_JAVA
@@ -1314,7 +1321,7 @@ public:
      * finalizer/destructor has run to notify helper threads that the
      * application is destroyed).
      */
-    operator bool() const { return ok_; }
+    explicit operator bool() const { return ok_; }
 
     /*! \brief Releases the lock.
      */
@@ -1326,6 +1333,16 @@ public:
     void release();
 #endif
 
+#ifdef WT_TARGET_JAVA
+    /*! \brief Releases the lock.
+     *
+     * Calls release()
+     *
+     * Implemented in order to support the AutoCloseable interface.
+     */
+    virtual void close();
+#endif // WT_TARGET_JAVA
+
   private:
 #ifdef WT_TARGET_JAVA
     UpdateLock(WApplication *app);
@@ -1841,7 +1858,7 @@ public:
    *
    * The integer parameter is the request size that was received in bytes.
    */
-  Signal<::int64_t>& requestTooLarge() { return requestTooLarge_; }
+  Signal< ::int64_t>& requestTooLarge() { return requestTooLarge_; }
 
   /** @name Global keyboard and mouse events
    */
@@ -1938,8 +1955,6 @@ public:
    */
   void setConfirmCloseMessage(const WString& message);
 
-  /*! \brief Sets the message for the user when the application was .
-   */
   void enableInternalPaths();
 
   // should we move this into an InternalPaths utility class / namespace ?
@@ -2017,8 +2032,8 @@ protected:
    *
    * - for having a single point for exception handling: while you may want
    *   to catch recoverable exceptions in a more appropriate place, general
-   *   (usually fatal) exceptions may be caught here. You will in probably
-   *   also want to catch the same exceptions in the application constructor
+   *   (usually fatal) exceptions may be caught here. You will probably
+   *   want to catch the same exceptions in the application constructor
    *   in the same way.
    * - you want to manage resource usage during requests. For example, at
    *   the end of request handling, you want to return a database session
@@ -2108,6 +2123,84 @@ protected:
    */
   virtual void unload();
 
+  /*! \brief Idle timeout handler
+   *
+   * \if cpp
+   * If <tt>idle-timeout</tt> is set in the configuration, this method is called when
+   * the user seems idle for the number of seconds set in <tt>idle-timeout</tt>.
+   * \elseif java
+   * If idle timeout is set in the configuration
+   * ({@link Configuration#setIdleTimeout(int)}), this
+   * method is called when the user seems idle for the number of seconds set as the
+   * idle timeout.
+   * \endif
+   *
+   * This feature can be useful in security sensitive applications
+   * to prevent unauthorized users from taking over the session
+   * of a user that has moved away from or left behind
+   * the device from which they are accessing the %Wt application.
+   *
+   * The default implementation logs that a timeout has occurred,
+   * and calls quit().
+   *
+   * This method can be overridden to specify different timeout behaviour,
+   * e.g. to show a dialog that a user's session has expired, or that
+   * the session is about to expire.
+   *
+   * \if cpp
+   *
+   * Example for an expiration dialog:
+   *
+   * \code
+   * class MyApplication : public Wt::WApplication {
+   * public:
+   *   MyApplication(Wt::WEnvironment &env)
+   *    : WApplication(env)
+   *   { }
+   *
+   * protected:
+   *   virtual void idleTimeout() override
+   *   {
+   *     if (idleTimeoutDialog_)
+   *       return; // Prevent multiple dialogs
+   *
+   *     idleTimeoutDialog_ = addChild(std::make_unique<Wt::WDialog>("Idle timeout"));
+   *     idleTimeoutDialog_->contents()->addNew<Wt::WText>("This session will automatically quit in 1 minute, "
+   *                                                       "press 'abort' to continue using the application");
+   *     auto btn = idleTimeoutDialog_->footer()->addNew<Wt::WPushButton>("abort");
+   *     btn->clicked().connect([this]{
+   *       removeChild(idleTimeoutDialog_.get());
+   *     });
+   *     auto timer = idleTimeoutDialog_->addChild(std::make_unique<Wt::WTimer>());
+   *     timer->setInterval(std::chrono::seconds{60});
+   *     timer->setSingleShot(true);
+   *     timer->timeout().connect([this]{
+   *       quit();
+   *     });
+   *     timer->start();
+   *     idleTimeoutDialog_->show();
+   *   }
+   *
+   * private:
+   *   Wt::Core::observing_ptr<Wt::WDialog> idleTimeoutDialog_;
+   * };
+   * \endcode
+   *
+   * \endif
+   *
+   * \note The events currently counted as user activity are:
+   *  - mousedown
+   *  - mouseup
+   *  - wheel
+   *  - keydown
+   *  - keyup
+   *  - touchstart
+   *  - touchend
+   *  - pointerdown
+   *  - pointerup
+   */
+  virtual void idleTimeout();
+
   /**
    * @brief handleJavaScriptError print javaScript errors to log file.
    * You may want to overwrite it to render error page for example.
@@ -2229,6 +2322,7 @@ private:
 
   EventSignal<> showLoadingIndicator_, hideLoadingIndicator_;
   JSignal<> unloaded_;
+  JSignal<> idleTimeout_;
 
   WContainerWidget *timerRoot() const { return timerRoot_; }
   WEnvironment& env(); // short-hand for session_->env()
@@ -2273,6 +2367,7 @@ private:
   void setExposeSignals(bool how) { exposeSignals_ = how; }
   bool exposeSignals() const { return exposeSignals_; }
   void doUnload();
+  void doIdleTimeout();
 
 #ifndef WT_TARGET_JAVA
   int startWaitingAtLock();
