@@ -316,6 +316,9 @@ public:
      *
      * The function will consider the first argument as the condition, and the
      * second argument as the key for a localized string that is a macro block.
+     *
+     * Just like the block() function, you can provide additional arguments, so the third
+     * argument will be what is filled in into <tt>{1}</tt> in the macro block, etc.
      */
 #ifndef WT_TARGET_JAVA
     static bool while_f(WTemplate *t, const std::vector<WString>& args,
@@ -466,12 +469,17 @@ public:
   template <typename Widget>
     Widget *bindWidget(const std::string& varName,
                        std::unique_ptr<Widget> widget)
+#ifndef WT_TARGET_JAVA
   {
     Widget *result = widget.get();
     bindWidget(varName, std::unique_ptr<WWidget>(std::move(widget)));
     return result;
   }
+#else // WT_TARGET_JAVA
+  ;
+#endif // WT_TARGET_JAVA
 
+#ifndef WT_TARGET_JAVA
   /*! \brief Creates a new widget with the given arguments, and binds it, returning a raw pointer.
    *
    * This is implemented as:
@@ -494,6 +502,9 @@ public:
     bindWidget(varName, std::unique_ptr<WWidget>(std::move(w)));
     return result;
   }
+#endif // WT_TARGET_JAVA
+
+  using WWidget::removeWidget;
 
   /*! \brief Unbinds a widget by variable name.
    *
@@ -815,7 +826,7 @@ protected:
 private:
   typedef std::map<std::string, Function> FunctionMap;
   typedef std::map<std::string, WString> StringMap;
-  typedef std::map<std::string, std::unique_ptr<WWidget>> WidgetMap;
+  typedef std::map<std::string, std::unique_ptr<WWidget> > WidgetMap;
   typedef std::set<std::string> ConditionSet;
 
   std::set<WWidget *> *previouslyRendered_;
